package com.energy.online.data.service.listener;

import com.energy.online.data.dto.BaseConfig;
import com.energy.online.data.dto.WindFarm;
import org.openmuc.j60870.ClientConnectionBuilder;
import org.openmuc.j60870.Connection;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.IOException;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.util.HashMap;
import java.util.Map;
import java.util.concurrent.Callable;


public class JClient implements Callable<String> {

    private static final Logger log = LoggerFactory.getLogger(JClient.class);

    public static String title = "104协议数据服务 - ";

    private final WindFarm windFarm;

    private final BaseConfig baseConfig;

    //监听类，后面代码会给出
    private J60870ClientListener listener;
    //连接类
    private Connection connection;

    //连接参数，可以配置的参数有好多，比如t0,t1,t2等等
    private ClientConnectionBuilder clientConnectionBuilder;

    @Override
    public String call() throws Exception {
        while (!Thread.currentThread().isInterrupted()) {
            try {
                // 使用Builder创建连接
                connection = clientConnectionBuilder.build();
                log.info(String.format("成功连接至%s:%s%n", windFarm.getIp(), windFarm.getPort()));

                // 配置数据监听
                Map<Long, Map<Integer, Object>> dataMap = new HashMap<>();
                listener = new J60870ClientListener(dataMap, windFarm, baseConfig);
                connection.startDataTransfer(listener);

                // 修改连接状态检查方式
                while (connection != null && !connection.isClosed()) {
                    Thread.sleep(1000);
                }

            } catch (IOException e) {
                log.info("连接异常:" + e.getMessage());
            } finally {
                closeConnection();
            }

            if (!Thread.currentThread().isInterrupted()) {
                log.info("1秒后尝试重连...");
                Thread.sleep(1000);
            }
        }
        return "连接任务终止";
    }

    public JClient(WindFarm windFarm, BaseConfig baseConfig) throws UnknownHostException {
        this.windFarm = windFarm;
        this.baseConfig = baseConfig;
        init();
    }

    /**
     * 初始化
     */
    public void init() throws UnknownHostException {

        //获取server端、从站的ip地址对象，这里使用本地ip
        InetAddress address = InetAddress.getByName(windFarm.getIp());
        //创建连接参数对象
        clientConnectionBuilder = new ClientConnectionBuilder(address);

        //设置socket的连接超时时间
        clientConnectionBuilder.setConnectionTimeout(1000);

        //设置socket的连接端口
        clientConnectionBuilder.setPort(windFarm.getPort());

        log.info(title + "J60870Client Init Success By Host " + windFarm);
    }

    private void closeConnection() {
        if (connection != null && !connection.isClosed()) {
            connection.close();
            log.info("连接已正常关闭");
        }
    }
}
