package com.energy.online.data.service.listener;

import com.energy.online.data.dto.BaseConfig;
import com.energy.online.data.dto.WindFarm;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.annotation.PreDestroy;
import java.net.UnknownHostException;
import java.util.List;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Future;
import java.util.concurrent.TimeUnit;

import static java.util.concurrent.Executors.newFixedThreadPool;

public class ListenerMainService {

    private static final Logger log = LoggerFactory.getLogger(ListenerMainService.class);

    private ExecutorService executor = null;
    private final List<Future<?>> futures = new CopyOnWriteArrayList<>();

    public void onApplicationEvent(BaseConfig baseConfig) {
        log.info("启动监听到 J60870Client...");
        List<WindFarm> windFarms = baseConfig.getWindFarms();
        int taskNum = windFarms.size();

        // 修正线程池大小（确保不小于任务数量）
        int poolSize = Math.max(Runtime.getRuntime().availableProcessors() * 2, taskNum);
        executor = newFixedThreadPool(poolSize);

        for (WindFarm windFarm : windFarms) {
            log.info("Task " + windFarm);
            Future<String> future = null;
            try {
                future = executor.submit(new JClient(windFarm, baseConfig));
            } catch (UnknownHostException e) {
                e.printStackTrace();
                log.error(e.getMessage());
            }
            futures.add(future);
            log.info("监听地址：" + windFarm.getIp());
        }
    }


    @PreDestroy
    public void shutdown() {
        futures.forEach(future -> {
            if (!future.isDone()) {
                future.cancel(true);
            }
        });
        executor.shutdown();
        try {
            if (!executor.awaitTermination(5, TimeUnit.SECONDS)) {
                executor.shutdownNow();
            }
        } catch (InterruptedException e) {
            executor.shutdownNow();
            Thread.currentThread().interrupt();
        }
    }
}

